#!/usr/bin/env python
# -*- coding: utf8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import glob
import re

# Start sos compatibility code part 1
try:
    # Try to load SOS 3 API
    from sos.plugins import Plugin, RedHatPlugin
except ImportError:
    # Failed import, revert to 1.x/2.2 SOS API: alias the Plugin and
    # RedHatPlugin classes to the old PluginBase class
    import sos.plugintools
    Plugin = RedHatPlugin = sos.plugintools.PluginBase
# End sos compatibility code part 1


# Derive our plugin from the Dummy one
class ovm3(Plugin, RedHatPlugin):
    """ Gather system information for OVS systems
    """
    
    plugin_name = 'ovm3'
    version = '1.1.1'

    option_list = [
        ("no_logs", "Do not gather all logs from /var/log", "", False),
        ("lsof", "Run lsof (slow)", "", False),
    ]

    # Start sos compatibility code part 2
    # If we are in pre - 3.0 API, make aliases to old calls
    def addCopySpecCompat(self, copyspecs):
        """ Older API does not support copyspecs lists
        """
        if not isinstance(copyspecs, list):
            copyspecs = [copyspecs]
        for copyspec in copyspecs:
            Plugin.addCopySpec(self, copyspec)

    if not hasattr(Plugin, 'add_copy_spec'):
        add_copy_spec = addCopySpecCompat
        add_copy_spec_limit = Plugin.addCopySpecLimit
        add_cmd_output = Plugin.collectExtOutput
        if hasattr(Plugin, 'getOption'):
            # SOS 2.x
            get_option = Plugin.getOption
        else:
            # SOS 1.7
            get_option = Plugin.isOptionEnabled
        # Comment out next line if you don't have any option
        optionList = option_list

    # Comment out next method if you don't have a check_enabled() method:
    def checkenabled(self):
        return self.check_enabled()
    # End sos compatibility code part 2

    def check_enabled(self):
        return self.is_ovm_manager() or self.is_ovm_server() or self.is_pca()

    def is_ovm_manager(self):
        """
        Returns true value if we are on an Oracle VM Manager
        """
        return os.path.exists("/u01/app/oracle/ovm-manager-3")

    def is_ovm_server(self):
        """
        Returns true value if we are on an Oracle VM Server
        """
        return os.path.exists("/etc/ovs-release")

    def is_pca_manager(self):
        """
        Returns true value if we are on an Oracle PCA Manager node
        """
        return os.path.exists("/usr/sbin/pca-admin")

    def is_pca_server(self):
        """
        Returns true value if we are on an Oracle PCA Compute node
        """
        return glob.glob("/usr/share/doc/ovca-ovs-release-*/LICENSE")

    def is_pca(self):
        """
        Returns true if we are on an Oracle PCA
        """
        return self.is_pca_manager() or self.is_pca_server()

    def setup(self):
        """
        Data collection
        """

        # Data we gather on all systems:
        #
        # All log files
        if not self.get_option('no_logs'):
            # By default we gather all logs in var log
            self.add_copy_spec("/var/log")
        self.add_copy_spec("/var/log/messages*")
        # Overall system info
        self.add_copy_spec("/root/anaconda-ks.cfg")
        self.add_cmd_output("/bin/cat /root/.bash_history", suggest_filename="bash_history")
        # NTP
        self.add_copy_spec("/etc/ntp.conf")
        self.add_cmd_output("/sbin/service ntpd status", suggest_filename="ntpd.status")
        self.add_cmd_output("/usr/sbin/ntpq -pn", suggest_filename="ntpq_-pn")
        self.add_cmd_output("/usr/bin/pgrep ntpd", suggest_filename="pgrep_ntpd")
        self.add_cmd_output("/bin/sh -c 'netstat -tulpn | grep 123'", suggest_filename="netstat_-tulpn_grep_123")
        self.add_cmd_output("/bin/grep -i ntp /var/log/messages", suggest_filename="ntp_messages")
        # Network info
        self.add_cmd_output("/sbin/ip -s -s -s link show", suggest_filename="ip-s-s-s-link-show")
        self.add_cmd_output("/sbin/ip address show", suggest_filename="ip-address-show")
        self.add_cmd_output("/sbin/ifconfig -a", suggest_filename="ifconfig")
        self.add_cmd_output("/bin/netstat -lntu", suggest_filename="netstat_open_ports")
        self.add_cmd_output("/bin/netstat -i", suggest_filename="netstat_i")
        self.add_cmd_output("/bin/sh -c 'ip route | awk \"/default/ { print \\$3 }\" | xargs ping -c 3'", suggest_filename="ping_default_gw")
        # Process info
        self.add_cmd_output("/bin/ps -elfy", suggest_filename="ps-elfy")
        self.add_cmd_output("/usr/bin/top -b -n 1", suggest_filename="top")
        # Installed packages
        self.add_cmd_output("/bin/rpm -qa --queryformat '%{installtime} %{name}-%{version}-%{release} %{installtime:date}\n'", suggest_filename="rpm_qa_installtime")
        # Run vmpstat
        if os.path.exists("/usr/sbin/vmpstat.sh"):
            self.add_cmd_output("/usr/sbin/vmpstat.sh -S", suggest_filename="vmpstat.out")

        # Only on Manager nodes
        #
        if self.is_ovm_manager():
            self.add_cmd_output("/bin/cat /u01/app/oracle/ovm-manager-3/.config", suggest_filename="ovm-manager_3_config")

        # Only on Server nodes
        #
        if self.is_ovm_server():
            # Overall node info
            self.add_cmd_output("/bin/cat /etc/ovs-config", suggest_filename="ovsconfig")
            self.add_cmd_output("/bin/cat /etc/ovs-info", suggest_filename="ovsinfo")
            self.add_cmd_output("/bin/cat /etc/ovs-release", suggest_filename="ovsrelease")
            # Misc log files (storage plugin, console plugin, ...)
            self.add_copy_spec([
                "/var/log/ovmwatch.lo*",
                "/var/log/osc.lo*",
                "/var/log/ovm-consoled.lo*",
                "/var/log/brcm-iscsi.lo*"
            ])

            # Repositories
            for vm in glob.glob('/OVS/Repositories/*/.ovsrepo'):
                self.add_cmd_output("/bin/cat " + vm, suggest_filename=vm)
            # Lock files if any
            self.add_copy_spec("/var/run/ovs-agent/*.lock*")
            # osmd files (as from ovm 3.4)
            if os.path.exists("/usr/sbin/osmdaemon"):
                self.add_copy_spec([
                    "/etc/sysconfig/osmd",
                    "/var/log/osmd"
                ])
            # NFS mounts
            self.add_cmd_output('/bin/sh -c \'/bin/mount | egrep -e "automount|pid.*nfs" 2>&1\'', suggest_filename="auto_nfs_check")
            # Xen
            # Check for toolstack
            xend_running = False
            ts_exec = None
            try:
                xend_status = os.popen('/etc/init.d/xend status').read().strip('\n')
            except Exception as e:
                print("Error running xend status = %s" % str(e))
                
            xendObj = re.match('.*\(pid\s(\d*)\).*', xend_status)
            if xendObj:
                proc_file = '/proc/' + xendObj.group(1)
                if os.path.exists(proc_file):
                    xend_running = True
            if os.path.exists("/usr/sbin/xl") and not xend_running:
                ts_exec = "/usr/sbin/xl "
                ts_prefix = "xl."
            elif os.path.exists("/usr/sbin/xm"):
                ts_exec = "/usr/sbin/xm "
                ts_prefix = "xm."
            else:
                noxen_msg = 'echo "No Xen toolstack found."'
                self.add_cmd_output(noxen_msg, suggest_filename="xen_not_found")
            if ts_exec is not None:
                self.add_cmd_output(ts_exec + "info", suggest_filename=ts_prefix + "info")
                self.add_cmd_output(ts_exec + "info -n", suggest_filename=ts_prefix + "info_n")
                self.add_cmd_output(ts_exec + "list", suggest_filename=ts_prefix + "list")
                self.add_cmd_output(ts_exec + "list -l", suggest_filename=ts_prefix + "list-l")
                self.add_cmd_output(ts_exec + "vcpu-list", suggest_filename=ts_prefix + "vcpu-list")
                self.add_cmd_output(ts_exec + "sched-credit", suggest_filename=ts_prefix + "sched-credit")
                self.add_cmd_output(ts_exec + "dmesg", suggest_filename=ts_prefix + "dmesg")
                for key in ('0','D','H','I','M','Q','V','a','c','d','e','g','i','m','s','u','x','z'):
                    outfile = ts_prefix + 'debug-keys-' + key
                    msgfile = outfile + '-dmesg'
                    cmd = ts_exec + "debug-keys " + key
                    self.add_cmd_output(cmd, suggest_filename=outfile)
                    self.add_cmd_output(ts_exec + "dmesg", suggest_filename=msgfile)
            self.add_cmd_output("/usr/sbin/xenpm get-cpuidle-states", suggest_filename="xenpm.get-cpuidle-states")
            self.add_cmd_output("/usr/sbin/xenpm get-cpufreq-states", suggest_filename="xenpm.get-cpufreq-states")
            self.add_cmd_output("/bin/sh -c '/usr/sbin/xenpm get-cpuidle-states | grep total | uniq'", suggest_filename="get_cpuidle_states_uniq")
            self.add_cmd_output("/usr/sbin/xenpm get-cpufreq-para", suggest_filename="xenpm.get-cpufreq-para")
            self.add_cmd_output("/usr/bin/xenstore-ls", suggest_filename="xenstore-ls")
            self.add_cmd_output("/sbin/service xend status", suggest_filename="xend.status")
            # No dominfo / dom0memcontroller command in 3.2 - 3.4 -- this is 'old' VirtualIron code
            # self.add_cmd_output("dominfo -c -f", suggest_filename="dominfo-c-f")
            # self.add_cmd_output("dom0memcontroller --dumpMemInfo", suggest_filename="dom0memcontroller--dumpMemInfo")
            # All domu configs
            self.add_cmd_output('/bin/sh -c \'find /OVS -type f -name "*.cfg" | while read -r line; do echo "====================================";cat "$line"; echo "===================================="; done\'', suggest_filename="vms.configs")
            # All ovm related packages
            self.add_cmd_output("/bin/sh -c 'rpm -qa | grep -E \(xen\|ovs\|ocfs\) | sort'", suggest_filename="rpm_qa_xen_ovs_ocfs")
            # Misc
            self.add_cmd_output("/bin/sh -c '/bin/ls -l /OVS/Repositories/*/VirtualDisks'", suggest_filename="listvdisks")
            self.add_cmd_output("/bin/ls -l -R /poolfsmnt", suggest_filename="ls-l-R_poolfsmnt")
            self.add_cmd_output("/bin/ls -l -R /nfsmnt", suggest_filename="ls-l-R_nfsmnt")
            self.add_cmd_output("/bin/ls -l -R /dlm", suggest_filename="ls-l-R-dlm")
            # Network
            self.add_cmd_output("/sbin/ip route show", suggest_filename="ip_route_show")
            self.add_cmd_output("/sbin/ip -s neighbor list", suggest_filename="ip_-s_neighbor_list")
            self.add_cmd_output("/sbin/arp -na", suggest_filename="arp_na")
            self.add_cmd_output("/bin/cat /proc/net/arp", suggest_filename="cat_proc_net_arp")
            self.add_cmd_output("/usr/sbin/brctl show", suggest_filename="brctl.show")
            for bond in glob.glob('/proc/net/bonding/bond*'):
                self.add_cmd_output("/bin/cat " + bond, suggest_filename="proc_net_" + bond)
            self.add_cmd_output("/sbin/multipath -ll", suggest_filename="multipath-ll")
            self.add_cmd_output('/bin/sh -c \'find /sys/class/net/* -printf "%f\n"  | while read -r line; do echo "$line ======================="; ethtool -i "$line"; echo "====================================";done\'', suggest_filename="ethtool")
            self.add_cmd_output("/bin/sh -c 'shopt -s nullglob; for f in /sys/class/fc_remote_ports/rport-*/dev_loss_tmo; do d=$( dirname $f); echo $(basename $d):$(cat $d/node_name):$(cat $f); done'", suggest_filename="fc_remote_ports_dev_loss_tmo")
            # Files / storage
            if self.get_option('lsof'):
                self.add_cmd_output("/usr/sbin/lsof")
            self.add_cmd_output("/sbin/fdisk -l", suggest_filename="fdisk_l")
            # Ovs agent
            self.add_copy_spec("/var/log/ovs-agent.*")
            self.add_cmd_output("/usr/sbin/ovs-agent-db dump_db server", suggest_filename="dump_db_server")
            self.add_cmd_output("/usr/sbin/ovs-agent-db dump_db aproc", suggest_filename="dump_db_aproc")
            self.add_cmd_output("/usr/sbin/ovs-agent-db dump_db repository", suggest_filename="dump_db_repository")
            self.add_cmd_output("/usr/sbin/ovs-agent-db -c dump_db server_pool", suggest_filename="dump_db_serverpool")
            self.add_cmd_output("/usr/sbin/ovs-agent-db -c dump_db server_pool_servers", suggest_filename="dump_db_server_pool_servers")
            self.add_cmd_output("/usr/sbin/ovs-agent-db -c dump_db monitored_vms", suggest_filename="dump_db_monitored_vms")
            # OVS (3.4.X) has oswatcher
            oswconf = "/etc/sysconfig/oswatcher"
            if os.path.exists(oswconf):
                oswfile = open(oswconf, 'r')
                oswdir = None
                for line in oswfile.readlines():
                    if line.startswith("DATADIR"):
                        label, oswdir = line.split("=")
                oswfile.close()
                if oswdir:
                    self.add_copy_spec(oswdir)
            # OCFS2 health check
            if os.path.exists("/usr/sbin/ocfs2hc"):
                self.add_cmd_output("/usr/sbin/ocfs2hc --json-output", suggest_filename="ocfs2hc.json")
                self.add_cmd_output("/usr/sbin/ocfs2hc --validate-cluster --json-output", suggest_filename="ocfs2hc_validate.json")
                self.add_cmd_output("/usr/sbin/ocfs2hc --cluster-report --json-output", suggest_filename="ocfs2hc_report.json")

        # Only on PCA (Manager and Compute Nodes)
        #
        if self.is_pca():
            # Config
            self.add_copy_spec("/etc/oracle*")
            # Install logs
            self.add_copy_spec("/root/instal*")
            # Xsigo
            self.add_copy_spec("/var/log/xsigo/xsadmdsh.lo*")
            self.add_cmd_output("/bin/sh -c 'lsmod  | grep xsv'", suggest_filename="xsigo_kernel_modules")

        # Only on PCA Manager nodes
        #
        if self.is_pca_manager():
            # Config
            self.add_copy_spec("/etc/ovca*")
            # Logs
            self.add_copy_spec("/var/log/ovca*")
            self.add_copy_spec("/var/log/pca*")
            self.add_copy_spec("/tmp/OVMModelDump_*")
            self.add_copy_spec("/tmp/virtualdata.xml")
            # Lists mounts
            self.add_cmd_output("/bin/mount", suggest_filename="pca_mgr.mounts")
            self.add_cmd_output("/sbin/service xms status", suggest_filename="service_xms_status")
            # Services
            self.add_cmd_output("/sbin/service ovca status", suggest_filename="service_ovca_status")
            # PCA checks
            if os.path.exists("/usr/sbin/pca-check-master"):
                self.add_cmd_output("/usr/sbin/pca-check-master", suggest_filename="pca_check_master")
            if os.path.exists("/usr/sbin/pca-diag"):
                self.add_cmd_output("/usr/sbin/pca-diag version", suggest_filename="pca_diag_version")
            if os.path.exists("/usr/sbin/pca-updater"):
                self.add_cmd_output("/usr/sbin/pca-updater -m update -l", suggest_filename="pca_update_list")
                self.add_cmd_output("/usr/sbin/pca-updater -m update -l -v", suggest_filename="pca_update_list_verbose")
            if os.path.exists("/usr/sbin/pca-admin"):
                self.add_cmd_output("/usr/sbin/pca-admin show system-properties --json", suggest_filename="pca_admin_show_system_properties.json")
                self.add_cmd_output("/usr/sbin/pca-admin list update-task --json", suggest_filename="pca_admin_list_update_task.json")
                self.add_cmd_output("/usr/sbin/pca-admin list storage-cloud --json", suggest_filename="pca_admin_list_storage_cloud.json")
                self.add_cmd_output("/usr/sbin/pca-admin list task --json", suggest_filename="pca_admin_list_task.json")
                self.add_cmd_output("/usr/sbin/pca-admin list management-node --json", suggest_filename="pca_admin_list_management_node.json")
                self.add_cmd_output("/usr/sbin/pca-admin list opus-port --json", suggest_filename="pca_admin_list_opus_port.json")
                self.add_cmd_output("/usr/sbin/pca-admin list compute-node --json", suggest_filename="pca_admin_list_compute_node.json")
                self.add_cmd_output("/usr/sbin/pca-admin list network-cloud --json", suggest_filename="pca_admin_list_network_cloud.json")
                self.add_cmd_output("/usr/sbin/pca-admin list config-error --json", suggest_filename="pca_admin_list_config_error.json")
                self.add_cmd_output("/usr/sbin/pca-admin list wwpn-info --json", suggest_filename="pca_admin_list_wwpn_info.json")
                self.add_cmd_output('/bin/sh -c \'/usr/sbin/pca-admin list network | cut -d " " -f 1 | sed "1,/------------/d;/----------------/,\\$d" | (echo "list-network: {"; i=0; while read line; do if [ $i -eq 0 ]; then i=1; else echo ","; fi; echo "$line: "; pca-admin show network "$line" --json; done; echo "}" )\'', suggest_filename="pca_admin_list_networks.json")
                self.add_cmd_output("/bin/ls -l /u01/app/oracle/ovm-manager-3/machine1/base_adf_domain/config/config.xml", suggest_filename="config_xml_link")
                self.add_cmd_output('/bin/sh -c \'echo "ovm_manager_3_config";cat /u01/app/oracle/ovm-manager-3/.config | grep UUID  | cut -d"=" -f2;echo "sysconfig_ovmm";cat /etc/sysconfig/ovmm | grep UUID | cut -d"=" -f2;echo "nfs_shared_conf_uuid";cat /nfs/shared_storage/conf/UUID\'', suggest_filename="system_uuids_out")

        # Only on PCA Compute nodes
        #
        if self.is_pca_server():
            pass
