# vmpstat.sh
# 
# --> Overview
#
# VMPStat is a maintenance script that can perform a quick health assessment of
# a supported Oracle VM Server or Manager.
#
# Author: Tom Lisjac <tom.lisjac@oracle.com> Oracle VM Product Management
# Initial Version: June 4, 2014
#
# Copyright (C) 2014 Oracle Corporation.  All rights reserved.
#
# --> Operation
#
# The script has 3 basic modes of operation:
#
#  * Analyze: (-A) the existing system and provide a status and healthcheck summary
#    on the console. Providing the ovs-agent password with the -p switch checks it 
#    for correctness. Makes no modification to the existing system configuration
#
#  * Dump: (-D) key system configuration parameters to the console. Can be redirected 
#    to a file or piped through the more command. Makes no system modifications.
#
#  * Service Dump (-S) combines the -A and -D commands to provide a cumulative
#    diagnostic display on the console and writes this information to a text
#    file for remote analysis.
#
#
# --> Oracle Documentation
#
# Please see the following MOS KM note for further information about this script:
#
#  VMPStat: Analyze and healthcheck an Oracle VM Server or Manager (Doc ID 1933450.1)
#
# --> Script status information
readonly VERSION="1.1"
readonly REVISION="1"
readonly TESTSTAT=""
readonly LASTUPDATE="November 6, 2014"
readonly MAINTAINER="Tom Lisjac <tom.lisjac@oracle.com>" # Contact for bug reports and feature suggestions
readonly SUPPORTEDVERS="3.2 3.3 3.4"
#
# --> Fixed session and filename parameters
readonly SESSTIME="$(date +%F\ %H:%M:%S)"       # ISO8601 compliant timestamp for this session
readonly TMPDT=${SESSTIME/ /-}                  # Build a filename compliant timestamp  
readonly DT=${TMPDT//:/}                        # "Now" timestamp used throughout the script
readonly SHORTHOSTNAME=`hostname|cut -d'.' -f1` # Safe short hostname: fix for -s bug in in some distros
readonly ARCTAG="vmpstat"                       # Script name and tag used in filenames and session id's
readonly SESSIONID="$SHORTHOSTNAME-$DT-$ARCTAG" # Session ID that provides a unique script instance tag
readonly CMDLINE="$@"
readonly STARTINGDIR=`pwd`
readonly COMMANDLINE="$@"

# --> Application specfic constants
readonly BASEDIR="/root/$ARCTAG"      # Default base directory for all writes
readonly SCANROOT=$BASEDIR/$SESSIONID # Subdirectory to deposit all data for this session
readonly SERVICEDUMPFN="/root/$SESSIONID.txt" # Location for ServiceDump text file

readonly SYSCONFIG=etc/sysconfig      # Where to find the network configuration files
readonly OVSAGENT=etc/ovs-agent
readonly OCFS2CONF=etc/ocfs2
readonly ISCSICONF=var/lib
readonly SYSCTL=etc/sysctl.conf
readonly OVMMInfo="/u01/app/oracle/ovm-manager-3/.config" # Location of OVMM info
readonly OVMMSys="/etc/sysconfig/ovmm"                    # Yet another location for OVMM info
readonly OVSInfo="/etc/ovs-info"      # Info about current OVS release
readonly EXTDNSTEST="oracle.com"      # lookup target for external DNS resolution test
readonly KMNOTE="1933450.1"           # Applicable Oracle KM note for this script

# Default script settings
COMMAND=""          # Command to execute
PASSWORD=""         # Cache for a password per the -p option
COLOROFF=""         # Disable ANSI colors for -A

# --> Global Variables                    
OSTYPE=""                  # Linux, Solaris... etc
RELEASEVER=""              # X.X.X
BUILD=""                   # Number of the product build
MAJORRELEASE=""            # X.X
V3ROLE=""                  # Either OVS or OVMM... set by the ThisIs* id functions
ISSUPPORTED=""             # Non-blank if this is a supported system
AHEXIT="0"                 # Red flag error accumulator for AnalyzeHost 

EXITHANDLER="{ cd $STARTINGDIR; exit $EXITVAL; }"

trap "$EXITHANDLER" SIGINT SIGKILL EXIT

function DoExit() { # single point exit from the script
  EXITVAL=$1
  if [ -n "$2" ]; then echo; echo "$2"; echo; fi
  exit # passes control to the exit handler 
}

# # # # # # # # # #
# Global Utilities

function color() {
  if [ -n "$COLOROFF" ]; then return 0; fi
  if [ "$2" = "" ]; then ATTR="1"; else ATTR="$2"; fi
  case "$1" in
    green) C="32";; red) C="31";; yellow) C="33";; blue) C="34";; norm) ATTR="0"; C="39";;
    cyan) C="36";; white) C="37";; purple) C="35";; none) return 0;; *)  C="$1";;
  esac
  echo -en "\e[$ATTR;${C}m"
}

function colorout() { color $1; echo -ne "$2"; color norm; }
function coloroutnl() { colorout $1 "$2"; echo; }

function IsRoot() { [ "`id -u`" = "0" ]; }

function ServiceUp() { if ! service $1 status 2>/dev/null; then return 1; else return 0; fi } 2>&1 >/dev/null
function ServiceDown() { if ! service $1 status 2>/dev/null; then return 0; else return 1; fi } 2>&1 >/dev/null
function ServiceStatus() {
  echo -n "  $1 service is "; if ServiceDown $1; then coloroutnl red "down"; return 1; else coloroutnl green "active"; return 0; fi
}

function PackedServiceStatus() {
  echo -n "  "
  for i in $@; do
    service=`echo $i|cut -d',' -f1`; downcolor=`echo $i|cut -d',' -f2`
    echo -n "$service:"; if ServiceDown $service; then colorout $downcolor "down  "; else colorout green "active  "; fi
  done
  echo
}

function StartServices() {
  for i in $*; do  if ! service $i status 2>&1 >/dev/null; then service $i start; fi; done
}  

function StopServices() {
  for i in $*; do if service $i status 2>&1 >/dev/null; then service $i stop; fi; done
}

function GetPassword() { if [ -z "$PASSWORD" ]; then echo; read -s -p "Manager Password (no echo):" PASSWORD; echo; fi; }


# # # # # # # # # # # # #
# Main program components
#

function VerifyOp() { # Confirm permissions are ok for the operation and host is supported for it 
  if ! IsRoot; then echo "Command $COMMAND must be run as root"; echo; return 1; fi
  if [ -z "$ISSUPPORTED" ]; then
    echo "The operation ($COMMAND) is not supported on this system"; echo
    echo "Supported systems are Oracle VM Manager and Server $SUPPORTEDVERS"; echo
    echo "Please see Oracle KM note $KMNOTE for more information about this script"
    echo; return 1
  fi
  return 0
}

function Initialize() { # Identify the host type and set the appropriate variables
  
  function IsThisOVMM() {
    if [ -f $OVMMInfo ]; then
      V3ROLE="OVMM"
      RELEASEVER=`grep BUILDID= $OVMMInfo | cut -d'=' -f2`
      MAJORRELEASE=`grep BUILDID= $OVMMInfo | cut -d'=' -f2|cut -d'.' -f1,2`
      return 0
    fi   
  }

  function IsThisOVS() {
    if [ -f $OVSInfo ]; then
      V3ROLE="OVS"
      RELEASEVER=`cat $OVSInfo | grep -i release: | cut -d' ' -f2`
      BUILDID=`cat $OVSInfo | grep -i build: | cut -d' ' -f2`
      MAJORRELEASE=`echo $RELEASEVER|cut -d'.' -f1,2`
    fi
  }

  # Initialize

  # check for Linux host and see if it's ovs or ovmm
  ISSUPPORTED="" # Assume it's not Linux or a supported OVS or OVMM version
  V3ROLE=""        # Either OVMM or OVS
  OSTYPE=`uname`; 
  if [ "$OSTYPE" = "Linux" ]; then
    IsThisOVS; IsThisOVMM 
    if [ -n $V3ROLE ]; then
      for i in $SUPPORTEDVERS; do
        if [ "$i" = "$MAJORRELEASE" ]; then ISSUPPORTED="yes"; return 0; fi
      done
    fi
  fi
  return 1
}
  
function AnalyzeHost() { # Scan and report host status without making any modification... just informational

  function CheckOS() {
    coloroutnl white "Operating System:"
    echo -n "  Kernel release: "; uname --kernel-release
    rootspace=`df -h / | grep -v Filesystem|xargs|cut -d' ' -f2,3,4`
    bootspace=`df -h /boot | grep -v Filesystem|xargs|cut -d' ' -f2,3,4`
    swapspace=`free -m | grep Swap:|xargs`
    echo "  Filesystem usage: Root /:$rootspace  /boot:$bootspace  $swapspace"
    echo -n "  Memory MB: used=`free -m | grep Mem:| xargs | cut -d' ' -f3` free=`free -m | grep Mem:| xargs | cut -d' ' -f4`    "
    dstates=`ps -elf|cut -d' ' -f2|grep -ic D`
    if [ "$dstates" = "0" ]; then coloroutnl green "D-state processes: $dstates"; else coloroutnl red "D-state processes: $dstates"; fi
    echo -n "  Uptime: "; uptime
  }

  function CheckNTP() {
    coloroutnl white "NTP:"
    if ServiceStatus ntpd; then echo -n "  "; ntpstat|head -n1; fi
  }

  function CheckDNS() {
    coloroutnl white "DNS: (timeout = 1 second)"
    if ! rpm -q --quiet bind-utils; then echo "  DNS verification tools not available on this version"; return 0; fi
    hn=`hostname`
    if [ -z "$hn" ]; then coloroutnl red "  Hostname not set"
    else
      if host -W 1 $hn 2>&1 >/dev/null; then coloroutnl green "  `host -W 1 $hn|head -n1`"
      else coloroutnl red "  hostname: $hn does not resolve from any source"
      fi
    fi
    ns=`cat /etc/resolv.conf|grep ^nameserver|cut -d' ' -f2`
    if [ -z "$ns" ]; then coloroutnl red "  no nameservers found in /etc/resolv.conf"
    else
      echo -n "  Configured DNS servers that ping: "
      for i in $ns; do if ping -c 1 -w 2 $i 2>&1 >/dev/null; then colorout green "$i "; else colorout red "$i "; fi; done; echo
    fi
    echo -n "  External DNS resolution "
    if host -W 1 $EXTDNSTEST 2>&1 >/dev/null; then echo -n "is"; else echo -n "is not"; fi; echo " available" 
  }

  function CheckNetworking() {
  
    function DumpInterfaceData() {
      echo "  Interface status:"
      pushd . 2>&1 >/dev/null
      cd /etc/sysconfig/network-scripts
      for i in ifcfg-*; do
        intf=`echo $i|cut -d'-' -f2`
        unset ipaddr speed duplex
        # ipaddr="`ip addr | grep -A2 $intf:|grep "inet "|xargs|cut -d' ' -f2`" # ip addr lumps the interfaces and bonds :(
        ipaddr=`ifconfig $intf 2>/dev/null|grep "inet "|xargs|cut -d':' -f2|cut -d' ' -f1` # old, but reliable and easy to parse
        if echo $intf|grep -iq eth; then
          speed="`ethtool $intf | grep Speed:|xargs|cut -d' ' -f2`, "
          duplex="`ethtool $intf | grep Duplex:|xargs|cut -d' ' -f2` duplex, "
        fi
        if ip link show $intf| grep -q "state UP"; then link="yes"; else link="no"; fi
        bond=`grep MASTER $i| cut -d'=' -f2`
        echo -n "    $intf: $speed${duplex}Link:"; if [ "$link" = "yes" ]; then colorout green "yes"; else colorout yellow "no"; fi
        unset ONBOOT BOOTPROTO
        source /etc/sysconfig/network-scripts/$i
        if [ -z "$ipaddr" ]; then realip="ip:none"
        else
         if echo "$BOOTPROTO"|grep -qi dhcp; then realip="dhcp-ip=$ipaddr"; else realip="fixed-ip=$ipaddr"; fi
        fi
        echo -n " onboot:$ONBOOT, $realip"
        if [ -z "$bond" ]; then echo
        else
          unset IPADDR BOOTPROTO ONBOOT
          bondip=`ip addr | grep -A2 $bond:|grep "inet "|xargs|cut -d' ' -f2`
          source /etc/sysconfig/network-scripts/ifcfg-$bond
          if [ -z "$bondip" ]; then realip="ip:none"
          else
            if echo "$BOOTPROTO"|grep -qi dhcp; then realip="dhcp-ip=$bondip"; else realip="fixed-ip=$bondip"; fi
          fi
          echo " ($bond:$realip, onboot:$ONBOOT)" 
        fi
      done
      popd 2>&1 >/dev/null
    }
  
    # Begin CheckNetworking
    coloroutnl white "Host Networking:"
    DEFAULTGW=`route -n|grep ^0.0.0.0|xargs|cut -d' ' -f2`
    if [ -z $DEFAULTGW ]; then echo "  No default gateway set"
    elif ping -c 1 -w 2 $DEFAULTGW >/dev/null 2>&1; then coloroutnl green "  Default gateway $DEFAULTGW is pingable"
    else coloroutnl red "  Default gateway $DEFAULTGW does not ping"
    fi
    if iptables -L | egrep -v Chain\|target\|^$ 2>&1 >/dev/null; then coloroutnl yellow "  firewall rules are active"
    else coloroutnl yellow "  No firewall rules are active"
    fi 
    DumpInterfaceData   
  }

  function CheckLocalServicePorts() { # dump nc status of specified port ($1),service type ($2)
    echo -n "  Local ports: "
    for i in $@; do
      portnum=`echo $i|cut -d',' -f1`; portname=`echo $i|cut -d',' -f2`
      echo -n "$portnum ($portname):"; if nc -z 127.0.0.1 $portnum 2>&1 >/dev/null; then colorout green "up  "; else colorout red "down  "; fi
    done
    echo 
  }
  
  function AnalyzeOVS() { # Perform a scan of key OVS and OVMM operational parameters. Flag errors in red or warnings in yellow
  
    function CheckServerEnvironment() {
      coloroutnl white "Xen Environment:"
      totmem=`xm info | grep total_memory|xargs`
      freemem=`xm info | grep free_memory|xargs`
      nrcpus=`xm info | grep nr_cpus|xargs`
      freecpus=`xm info | grep free_cpus|xargs`
      cpumhz=`xm info | grep cpu_mhz|xargs`
      numvms=`xm list| grep -v 'Name\|Domain-0'|wc -l`
      echo "  $totmem  $freemem  $nrcpus  $freecpus  $cpumhz  Num VMs : $numvms"
      echo "  `xm info | grep xen_commandline|xargs`"
      coloroutnl white "OVS and misc services:"
      PackedServiceStatus network,red sshd,red crond,red iptables,yellow ocfs2,red o2cb,red
      PackedServiceStatus ovs-agent,red xend,red xendomains,red ovs-devmon,red
      PackedServiceStatus openvswitch,yellow multipathd,yellow iscsi,yellow iscsid,yellow nfs,yellow
    }
    
    function CheckOVSEnvironment() {
      coloroutnl white "OVS status:"
      clustate=`ovs-agent-db dump_db server | grep cluster_state | cut -d"'" -f4`
      clustered=`ovs-agent-db dump_db server | grep clustered | cut -d' ' -f3|cut -d',' -f1`
      ismaster=`ovs-agent-db dump_db server | grep is_master | cut -d' ' -f3|cut -d',' -f1`
      poolalias=`ovs-agent-db dump_db server | grep pool_alias | cut -d"'" -f4`
      mgruuid=`ovs-agent-db dump_db server | grep manager_uuid | cut -d"'" -f4`
      pooluuid=`ovs-agent-db dump_db server |grep poolfs_uuid | cut -d"'" -f4`
      echo "  is_master:$ismaster   clustered:$clustered   cluster_state:$clustate   pool_alias:$poolalias"
      echo "  Owning Manager UUID:$mgruuid   Pool UUID:$pooluuid"
      registeredip=`ovs-agent-db dump_db server | grep registered_ip | cut -d"'" -f4`
      registeredhostname=`ovs-agent-db dump_db server | grep registered_hostname | cut -d"'" -f4`
      roles=`ovs-agent-db dump_db server | grep roles | cut -d"(" -f2|cut -d')' -f1`
      echo "  registered_hostname:$registeredhostname   registered_ip:$registeredip   roles:$roles"
      if ! [ -f /etc/ovs-agent/db/server ]; then echo "  Server database for ovs-agent does not exist. No data available for analysis"; return 1; fi
      if [ -n "$PASSWORD" ]; then
        if ! ServiceUp ovs-agent; then coloroutnl red "  Cannot check OVS agent password: ovs-agent service is not running"
        else
          { ovsres=`ovs-agent-rpc -s https://oracle:$PASSWORD@127.0.0.1:8899/ echo "'$ARCTAG'"`;} 2>/dev/null
          if [ "$ovsres" = "$ARCTAG" ]; then coloroutnl green "  Provided OVS agent password is correct"
          else coloroutnl red "  Provided OVS agent password does not match"
          fi
        fi
      fi 
      if [ "$MAJORRELEASE" = "3.3" ]; then 
        MGRIP=`ovs-agent-db dump_db server|grep manager_event_url |xargs|cut -d'/' -f3 | cut -d':' -f1`
        MGRPORT=`ovs-agent-db dump_db server|grep manager_event_url |xargs|cut -d'/' -f3 | cut -d':' -f2`
      else
        MGRIP=`ovs-agent-db dump_db server|grep manager_core_api_url |xargs|cut -d'@' -f2|cut -d'/' -f1|cut -d':' -f1`
        MGRPORT=`ovs-agent-db dump_db server|grep manager_core_api_url |xargs|cut -d'@' -f2|cut -d'/' -f1|cut -d':' -f2`      
      fi
      if [ -z "$MGRIP" ]; then coloroutnl yellow "  No Manager for this node"
      elif nc -z $MGRIP $MGRPORT >/dev/null 2>&1; then coloroutnl green "  Manager at $MGRIP:$MGRPORT is reachable on the network"
      else coloroutnl red "  Manager at $MGRIP:$MGRPORT is not reachable on the network"
      fi
    }

    function CheckClusterConfig() {
      clustered=`ovs-agent-db dump_db server | grep clustered| cut -d' ' -f3|cut -d',' -f1`
      if [ "$clustered" != "True" ]; then echo "This node is not clustered"; return 1; fi
      POOLVIP=`ovs-agent-db dump_db server|grep pool_virtual_ip | cut -d':' -f2 | xargs| cut -d',' -f1`
      if [ -z "$POOLVIP" ]; then coloroutnl red "  No VIP listed in agent database"
      elif nc -z $POOLVIP 8899 >/dev/null 2>&1; then coloroutnl green "  VIP at $POOLVIP:8899 is reachable on the network"
      else coloroutnl red "  VIP at $POOLVIP:8899 is not reachable on the network"
      fi
      poolmnt=`mount | grep /poolfsmnt/ | cut -d' ' -f3`
      if [ -w $poolmnt ]; then coloroutnl green "  Pool filesystem is mounted and writeable" 
      else coloroutnl red "  Pool filesystem is not mounted or writeable"
      fi
    }

  
    function CheckRepoStorage() {
      coloroutnl white "OVS Repo Storage:"
      repos=`ovs-agent-db dump_db repository | grep mount_point| cut -d"'" -f4`
      repocount=0; error=""
      for i in $repos; do
        if ! mount|grep -q $i; then coloroutnl red "  Expected repo $i not mounted"; error=1
        elif ! [ -w $i ]; then  coloroutnl red "  Expected repo $i not writeable"; error=1 
        fi
        repocount=$((repocount+1))    
      done
      if [ -z "$error" ]; then coloroutnl green "  $repocount expected repo(s) mounted and in a writeable state."; fi
      actrepos=`mount| grep /OVS/Repositories|wc -l`
      if [ "$repocount" = "$actrepos" ]; then coloroutnl green "  $actrepos mounted repo(s) matches $repocount registered in ovs-agent"
      else coloroutnl red "  $actrepos mounted repo(s) does not match $repocount expected in ovs-agent"
      fi
    }

    function CheckOVSPeers() {
      local checkedports="7777 8002 8003 8899"
      colorout white "Pool Nodes responding on ports: $checkedports"
      clupeers=`grep ip_address /etc/ocfs2/cluster.conf| cut -d'=' -f2|xargs`
      if [ -z "$clupeers" ]; then coloroutnl yellow "  This node has no registered cluster peers"
      else
        count=0; ocfs2oops=""; netoops=""
        for i in $clupeers; do
          if (($((count%7)) == 0)); then echo; echo -n "  "; fi; count=$((count+1)) # 7 IP's per line
          pmsg=""; peeroops=""
          for p in $checkedports; do
            if ! nc -z -w1 $i $p 2>&1 >/dev/null; then 
              netoops="1"; peeroops="1";
              if [ -z "$pmsg" ]; then pmsg=":$p"; else pmsg="$pmsg,$p"; fi
            fi
          done
          if [ -z "$peeroops" ]; then colorout green "$i  "; else  colorout red "**$i$pmsg  "; fi        
        done
        echo
        if [ -n "$netoops" ]; then coloroutnl red "  Nodes with a \"**\" prefix did not respond on port(s) listed"; fi
      fi
    }

    # Begin AnalyzeOVS
    if VerifyOp; then
      CheckOS
      CheckNetworking
      CheckDNS
      CheckNTP
      CheckServerEnvironment
      if [ "$MAJORRELEASE" = "3.3" ]; then ovscon="9020,console 25,smtp"; else ovscon=""; fi
      CheckLocalServicePorts 22,sshd 111,portmap 8002,migrate 8003,migrate-ssl 
      CheckLocalServicePorts 8899,ovs-agent $ovscon
      if ! [ -f "/$OVSAGENT/db/server" ]; then 
        coloroutnl red "  /$OVSAGENT/db/server not found. System has either been damaged or needs a reboot after a clean operation"
      else
        CheckOVSEnvironment
        CheckClusterConfig
        CheckOVSPeers
        CheckRepoStorage
      fi
    fi
  }
  
  # AnalyzeOVMM
  # This format is available in /u01/app/oracle/ovm-manager-3/.config from 3.0.1 to 3.3.1 
  #   DBTYPE=MySQL DBHOST=localhost SID=ovs LSNR=49500 OVSSCHEMA=ovs APEX=8080 WLSADMIN=weblogic
  #   OVSADMIN=admin COREPORT=54321 UUID=0004fb000001000026778f17a0551bd8 BUILDID=3.3.1.1061

  function AnalyzeOVMM() {
  
    # Begin AnalyzeOVMM 
    CheckOS
    CheckNetworking
    CheckDNS
    CheckNTP
    coloroutnl white "OVMM and misc services:"
    local ovmmdb=""
    source $OVMMInfo
    if [ "$DBHOST" = "localhost" ]; then
      if [ $DBTYPE = "MySQL" ]; then ovmmdb="ovmm_mysql"; else ovmmdb="oracle"; fi
      if [ "$MAJORRELEASE" = "3.3" ]; then cliport="10000,ovmm-cli"; else cliport=""; fi  
      localports="$LSNR,db-listen $cliport"
    fi
    PackedServiceStatus ovmm,red ovmcli,yellow $ovmmdb,red
    PackedServiceStatus network,red sshd,yellow crond,red iptables,yellow
    CheckLocalServicePorts $COREPORT,core-api 7002,ovmm-ui $localports
    coloroutnl white "OVMM Environment: ($OVMMInfo and $OVMMSys)"
    echo "  Oracle VM Manager $BUILDID"
    echo "  Manager UUID=$UUID"
    echo "  DB Type=$DBTYPE  DB Host=$DBHOST  DB Port=$LSNR"
    source $OVMMSys
    echo "  RUN_OVMM=$RUN_OVMM  USETLS1=$USETLS1  JVM_MAX_PERM=$JVM_MAX_PERM  JVM_MEMORY_MAX=$JVM_MEMORY_MAX"
    echo -n "  WLS Memory: "; /u01/app/oracle/java/bin/jps -lmv | grep weblogic.Server | cut -d' ' -f3,4,5
  }

  # Begin AnalyzeHost
  if VerifyOp; then
    AHEXIT="0" # Global error flag set by any red flag condition
    coloroutnl white "Host Analysis for $V3ROLE $RELEASEVER $BUILDID on `hostname`"
    case $V3ROLE in
      "OVS")  AnalyzeOVS ;;
      "OVMM") AnalyzeOVMM ;;
    esac
    coloroutnl white "Host Analysis completed"; echo;
    if [ "$COMMAND" = "AnalyzeHost" ]; then echo "For a detailed console dump of host configuration data, run: $ARCTAG -D"; echo; fi
    return "$AHEXIT"
  fi
}

function DumpHostData() { # Dumps a pure text summary of all the key OVS and OVMM operating parameters to the console

  function DumpLinuxConfigData() {
    
    function DumpFile() { for i in $@; do echo "-->$i"; grep -v '^$\|^#' $i; echo; done; }

    # Begin DumpLinuxConfigData
    echo
    DumpFile /etc/oracle-release /etc/system-release /etc/resolv.conf /etc/hosts /etc/ntp.conf /etc/exports
    DumpFile /etc/fstab /etc/inittab /etc/sysctl.conf /etc/yum.conf
    echo "-->/etc/yum.repos.d"; ls -la /etc/yum.repos.d; echo
    if [ "$V3ROLE" = "OVS" ]; then
      DumpFile /etc/ovs-info /etc/ovs-release /etc/ovs-config /etc/ovm-consoled.conf /etc/ovs-agent/agent.ini
      DumpFile /etc/ocfs2/cluster.conf /etc/sysconfig/o2cb
    fi
    echo "-->/etc/ntp.conf"; grep -v '^$\|^#' /etc/ntp.conf; echo  #strip all the comments and blank lines
    DumpFile /etc/sysconfig/network
    DumpFile /etc/sysconfig/iptables 
    echo "-->ip addr"; ip addr; echo
    echo "-->ip route"; ip route; echo
    echo "-->brctl show"; brctl show; echo
    echo "-->Network Interfaces:"; echo
    for i in /etc/sysconfig/network-scripts/ifcfg-*; do echo "$i"; cat $i; echo; done
    echo "-->mount"; mount; echo
    echo "-->df -h"; df -h; echo
    echo "-->blkid"; blkid; echo
    echo "--> multipath -ll"; multipath -ll; echo
  }

  function DumpOVSData() {
    
    function DumpOVSDB() {
      echo;
      echo "# # # # # # # # # # # # # # #"
      echo "# Dump of ovs-agent $1 data"
      echo
      echo "-->/etc/ovs-agent/db/$1"
      if [ -f /etc/ovs-agent/db/$1 ]; then ovs-agent-db dump_db $1 
      else echo "File /etc/ovs-agent/db/$1 not found"; echo
      fi
    }
    
    function DumpXenData() {
      echo; echo "--> Xen Data"; echo
      echo "-->xm info"; xm info; echo
      echo "-->xm list"; xm list; echo
      echo "-->xentop -i1 -b"; xentop -i1 -b; echo
    }

    # Begin DumpOVSData
    echo "Key parameter dump from `hostname` $SESSTIME"; echo
    DumpLinuxConfigData
    DumpXenData
    DumpOVSDB server   
    DumpOVSDB repository
    DumpOVSDB exports
    DumpOVSDB aproc
    echo
  }
  
  function DumpOVMMData() {
    DumpLinuxConfigData
    DumpFile $OVMMInfo
    DumpFile $OVMMSys
  }

  # Begin DumpHostData
    if VerifyOp; then
      case $V3ROLE in
       "OVS")  DumpOVSData ;;
       "OVMM") DumpOVMMData ;;
      esac
    fi
}

function ServiceDump() { # Combines the -Ak and -D into a simultaneous dump to the console and file $SERVICEDUMPFN
  COLOROFF="1"
  { AnalyzeHost; echo; DumpHostData; } | tee -a $SERVICEDUMPFN
  echo
  coloroutnl white "Service Dump completed to file $SERVICEDUMPFN";
  echo
} 

function Usage() {
  echo
  echo "$ARCTAG provides a health assessment, post-install reset and backup/restore of" 
  echo "a supported ($SUPPORTEDVERS) Oracle VM Server or Manager."
  echo
  echo "usage: $ARCTAG.sh COMMAND [OPTION]"
  echo
  echo "Options:"
  echo
  echo " -A  Analyze the host to determine its state. Makes no changes"
  echo " -D  Dump static system configuration data. Makes no changes"
  echo " -k  Disable onscreen ANSI colors for -A"
  echo " -p  OVS agent password to check ovs-agent pw, if provided with -A"
  echo " -S  Service dump of -Ak and -D diagnostic data to the console and a file"
  echo " -V  Show $ARCTAG version"
  echo
  echo "Examples for both OVS and OVMM:"
  echo
  echo "$ARCTAG.sh -A            Analysis and brief summary of the active system status"
  echo "$ARCTAG.sh -Ak           No ANSI escapes for capturing a pure text file"
  echo "$ARCTAG.sh -Ap hello123  Analysis and ovs-agent password of \"hello123\""
  echo "$ARCTAG.sh -S            Outputs -Ak and -D to the console and \<sessionID\>.txt"
  echo "$ARCTAG.sh -D            Dump OVM related system information and key OS settings" 
  echo
}

function ShowVersion() { coloroutnl white "$ARCTAG $VERSION.$REVISION $TESTSTAT  $1"; echo; }

function SetCommand() { if [ -z $COMMAND ]; then COMMAND=$1; else DoExit 2 "Only one command can be performed per session"; fi; }

while getopts "ADHhkp:SV" opt; do
  case "$opt" in
    A)  SetCommand AnalyzeHost ;;  # Just an Analysis... don't make any changes
    D)  SetCommand DumpHostData;;  # Dump host data... no changes
    k)  COLOROFF="1";;             # Disable ANSI colors for -A command
    p)  PASSWORD=$OPTARG ;;        # Password for checking the ovs-agent pw
    S)  SetCommand ServiceDump;;   # Combines -Ak and -D for a complete text dump to the console and $SERVICEDUMPFN
    V)  ShowVersion; DoExit 0;;    # Show version
 [hH])  Usage; DoExit 0;;          # Show help usage 
   \?) Usage "Error: $0 $*\n\n"; DoExit 1;;
   * ) Usage "Error: $0 $*\n\n"; DoExit 1;;
  esac
done
shift $(($OPTIND - 1))

# Main
clear; ShowVersion "SessionID: $SESSIONID  "
if [ -z "$COMMAND" ]; then Usage; DoExit 7; fi
Initialize
$COMMAND
DoExit $EXITVAL


%changelog
* Thu Nov 6 Tom Lisjac <tom.lisjac@oracle.com> 1.1.1
- Removed problematic logic around cluster port tests

* Tue Oct 27 Tom Lisjac <tom.lisjac@oracle.com> 1.1
- Added multipath -ll to dump bundle
- Added checks for 8002, 8003 and 7777 to -A peer network analysis
- Removed unused variables from initial vmpmaint cut and paste
- Renamed ROLE var to V3ROLE to avoid conflict in vmpscan

* Wed Oct 9 Tom Lisjac <tom.lisjac@oracle.com> 1.0.0
- Fork of just -A and -D commands from vmpmaint

